svn commit: trunk/uClibc/libc/sysdeps/linux/common

carmelo at uclibc.org carmelo at uclibc.org
Thu Sep 18 15:06:24 UTC 2008


Author: carmelo
Date: 2008-09-18 08:06:24 -0700 (Thu, 18 Sep 2008)
New Revision: 23429

Log:
Fix posix_fadvise[64] functions to return the error number in case
of failure instead of -1 and setting errno, according to SuSv3
(IEEE Std 1003.1 2004 edition) specification.

Signed-off-by: Filippo Arcidiacono <filippo.arcidiacono at st.com>
Signed-off-by: Carmelo Amoroso <carmelo.amoroso at st.com>


Modified:
   trunk/uClibc/libc/sysdeps/linux/common/posix_fadvise.c
   trunk/uClibc/libc/sysdeps/linux/common/posix_fadvise64.c


Changeset:
Modified: trunk/uClibc/libc/sysdeps/linux/common/posix_fadvise.c
===================================================================
--- trunk/uClibc/libc/sysdeps/linux/common/posix_fadvise.c	2008-09-18 13:16:03 UTC (rev 23428)
+++ trunk/uClibc/libc/sysdeps/linux/common/posix_fadvise.c	2008-09-18 15:06:24 UTC (rev 23429)
@@ -33,9 +33,19 @@
     return 0;
 }
 #else
-_syscall4(int, posix_fadvise, int, fd, off_t, offset,
-          off_t, len, int, advice);
+static __inline__ int syscall_posix_fadvise(int fd, off_t offset1, off_t offset2, off_t len, int advice);
+#define __NR_syscall_posix_fadvise __NR_fadvise64
+_syscall5(int, syscall_posix_fadvise, int, fd, off_t, offset1,
+          off_t, offset2, off_t, len, int, advice);
 
+int posix_fadvise(int fd, off_t offset, off_t len, int advice)
+{
+	int ret = syscall_posix_fadvise(fd, __LONG_LONG_PAIR (offset >> 31, offset), len, advice);
+	if (ret == -1)
+		return errno;
+	return ret;
+}
+
 #endif
 
 #if defined __UCLIBC_HAS_LFS__ && (!defined __NR_fadvise64_64 || !defined _syscall6)

Modified: trunk/uClibc/libc/sysdeps/linux/common/posix_fadvise64.c
===================================================================
--- trunk/uClibc/libc/sysdeps/linux/common/posix_fadvise64.c	2008-09-18 13:16:03 UTC (rev 23428)
+++ trunk/uClibc/libc/sysdeps/linux/common/posix_fadvise64.c	2008-09-18 15:06:24 UTC (rev 23429)
@@ -40,14 +40,35 @@
   return INTERNAL_SYSCALL_ERRNO (ret, err);
 }
 #else
-_syscall4(int, posix_fadvise64, int, fd, __off64_t, offset,
+static __inline__ int syscall_posix_fadvise(int fd, off_t offset1, off_t offset2, off_t len, int advice);
+#define __NR_syscall_posix_fadvise64 __NR_posix_fadvise64
+_syscall4(int, syscall_posix_fadvise64, int, fd, __off64_t, offset,
           __off64_t, len, int, advice);
+int posix_fadvise64(int fd, __off64_t offset, __off64_t len, int advice)
+{
+	int ret = syscall_posix_fadvise64(fd, offset, len, advice);
+	if (ret == -1)
+		return errno;
+	return ret;
+}
 #endif
 
 /* 32 bit implementation is kind of a pita */
 #elif __WORDSIZE == 32
 
-#ifdef _syscall6 /* workaround until everyone has _syscall6() */
+#ifdef INTERNAL_SYSCALL
+int posix_fadvise64(int fd, __off64_t offset, __off64_t len, int advice)
+{
+	INTERNAL_SYSCALL_DECL (err);
+	int ret = INTERNAL_SYSCALL (fadvise64_64, err, 6, fd,
+								__LONG_LONG_PAIR(offset >> 32, offset &  0xffffffff),
+								__LONG_LONG_PAIR(len >> 32, len & 0xffffffff),
+								advice);
+	if (!INTERNAL_SYSCALL_ERROR_P (ret, err))
+		return 0;
+	return INTERNAL_SYSCALL_ERRNO (ret, err);
+}
+#elif defined _syscall6 /* workaround until everyone has _syscall6() */
 #define __NR___syscall_fadvise64_64 __NR_fadvise64_64
 static __inline__ _syscall6(int, __syscall_fadvise64_64, int, fd,
           unsigned long, high_offset, unsigned long, low_offset,
@@ -55,14 +76,17 @@
           int, advice);
 int posix_fadvise64(int fd, __off64_t offset, __off64_t len, int advice)
 {
-	return (__syscall_fadvise64_64(fd,
+	int ret = __syscall_fadvise64_64(fd,
 	        __LONG_LONG_PAIR(offset >> 32, offset &  0xffffffff),
 	        __LONG_LONG_PAIR(len >> 32, len & 0xffffffff),
-	        advice));
+	        advice);
+	if (ret == -1)
+		return errno;
+	return ret;
 }
 #else
 #warning _syscall6 has not been defined for your machine :(
 #endif /* _syscall6 */
 
 #else
 #error your machine is neither 32 bit or 64 bit ... it must be magical
@@ -55,14 +76,17 @@
 }
 #else
-#warning _syscall6 has not been defined for your machine :(
-#endif /* _syscall6 */
+#warning neither INTERNAL_SYSCALL nor _syscall6 has been defined for your machine :(
+#endif /* INTERNAL_SYSCALL */
 
 #else




More information about the uClibc-cvs mailing list